#!/usr/bin/python3
# -*- codeing: UTF-8 -*-

import logging
import sys
import os
import getopt
import xml.dom.minidom as XP
import xlwt

def usage(appname):
    print('Usage:' + appname + '[OPTIONS] SRC DST\n' +
                '\t-t [file|dir]\t\t Convert Type\n' +
                '\t-h           \t\t Help info\n')
    sys.exit(0)
   
def make_style(isBold, fontSize, fontColour, backgroundColour):
    style=xlwt.XFStyle()

    font=xlwt.Font()
    font.charset=font.CHARSET_ANSI_CHINESE_GBK
    font.name='宋体'
    font.family=font.FAMILY_ROMAN
    font.escapement=font.ESCAPEMENT_NONE
    font.height=fontSize
    font.bold=isBold
    font.colour_index=fontColour
    style.font=font

    pattern=xlwt.Pattern()
    pattern.pattern=pattern.SOLID_PATTERN
    pattern.pattern_fore_colour=backgroundColour
    style.pattern = pattern

    border=xlwt.Borders()
    border.bottom=border.THIN
    border.left=border.THIN
    border.right=border.THIN
    border.top=border.THIN
    style.borders=border
    
    protect=xlwt.Protection()
    protect.cell_locked=True
    style.protection=protect
    
    return style
    
style_Header=make_style(True, 220, 0x09, 0x30)
style_0=make_style(False, 200, 0x08, 0x16)
style_1=make_style(False, 200, 0x08, 0x09)
style=[style_1, style_0]

def translate_xml_to_xlsx(src, dst):
    Dom=XP.parse(src)
    RootElement=Dom.documentElement
    if RootElement.localName != 'results':
        logging.info(src + ' is unexpect XML format')
        return

    eb=xlwt.Workbook()
    es=eb.add_sheet('ScanResult');
    header=False
    line=0

    #统计每列最长的宽度，设定宽度;GBK硬编码，修改style.font需要联动。。。
    ins_header='处理结果'
    ins_data='未处理'
    max_column_len=[len(ins_header.encode('GBK'))]
    line_in_column=4
    ErrElements=RootElement.getElementsByTagName("error")
    for Err in ErrElements:
        attrs=Err.attributes
        num=attrs.length
        i = 0
        sindex = line % 2
        if False == header:
            es.write(0, 0, ins_header, style_Header)
            es.write(1, 0, ins_data, style[1])
            while (i < num):
                es.write(0, i+1, attrs.item(i).name, style_Header)
                hlen=len(attrs.item(i).name.encode('GBK'))
                if attrs.item(i).name == 'line':
                    line_in_column = i
                    es.write(1, i + 1, int(attrs.item(i).value), style[1])
                else:
                    es.write(1, i + 1, attrs.item(i).value, style[1])
                vlen=len(attrs.item(i).value.encode('GBK'))
                if(hlen > vlen):
                    max_column_len.append(hlen)
                else:
                    max_column_len.append(vlen)
                i = i + 1
            header = True
            line=2
        else:
            es.write(line, 0, ins_data, style[sindex])
            while (i < num):
                if i==line_in_column:
                    es.write(line, i + 1, int(attrs.item(i).value), style[sindex])
                else:
                    es.write(line, i+1, attrs.item(i).value, style[sindex])
                if((i + 1) < len(max_column_len)):
                    vlen=len(attrs.item(i).value.encode('GBK'))
                    if(vlen > max_column_len[i+1]):
                        max_column_len[i+1] = vlen
                else:
                    logging.debug(attrs.item(i).name + '=' + attrs.item(i).value)
                i = i + 1
            line=line + 1

    if True==header:
        column_num=len(max_column_len)
        i = 0
        while(i < column_num):
            width=256*(max_column_len[i]+2)
            if width>65536:
                es.col(i).set_width(256*16)
            else:
                es.col(i).set_width(width)
            i = i + 1
        eb.save(dst)
            

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    type=''
    try:
        opts, args = getopt.getopt(sys.argv[1:], "t:h")
    except getopt.GetoptError:
        usage(sys.argv[0])

    for opt, arg in opts:
        if opt == '-t':
            type=arg
        elif opt == '-h':
            usage(sys.argv[0])
    
    if len(args) != 2:
        usage(sys.argv[0])

    if type == 'file':
        if False == os.access(args[0], os.F_OK):
            logging.error(args[0] + 'not exsit')
            sys.exit(0)
        if False == os.access(args[0], os.R_OK):
            logging.error(args[0] + 'not permission')
            sys.exit(0)
        if True == os.access(args[1], os.F_OK):
            if True == os.path.isfile(args[1]):
                logging.warning(args[1] + 'overwrite')
            else:
                logging.error(args[1] + 'exsit but not a file')
                sys.exit(0)

        dir=os.path.dirname(args[1])
        if False == os.access(dir, os.F_OK):
            logging.error(dir + 'not exsit')
            sys.exit(0)
        if False == os.access(dir, os.W_OK):
            logging.error(dir + 'not write permission')
            sys.exit(0)

        if True == os.path.isfile(dir):
            logging.error(dir + 'is file')
            sys.exit(0)

        translate_xml_to_xlsx(args[0], args[1])
    elif type == 'dir':
        print("Unsuppourt")
    else:
        usage(sys.argv[0])


